use color_eyre::eyre;


fn distance_between(a: usize, b: usize) -> usize {
    // can't do the fancypants (a - b).abs() thing here because these are unsigned
    if a > b {
        a - b
    }
    else {
        b - a
    }
}


fn cost_simple(counts: &[usize], pos: usize) -> usize {
    counts.iter()
        .enumerate()
        .fold(0, |total, (i, count)| {
            total + (distance_between(pos, i) * count) // yes, these parens are unnecessary
        })
}


fn cost_complex(counts: &[usize], pos: usize) -> usize {
    counts.iter()
        .enumerate()
        .fold(0, |total, (i, count)| {
            let dx = distance_between(pos, i);
            let cost_single = dx * (dx + 1) / 2; // Gauss' method for summing numbers from 1 to n
            total + (cost_single * count)
        })
}


pub fn run(data: &str) -> eyre::Result<(usize, usize)> {
    let mut counts = Vec::new();
    for s in data.trim().split(',') {
        let n = s.parse::<usize>()?;
        if n >= counts.len() {
            counts.resize(n + 1, 0);
        }
        counts[n] += 1;
    }

    let mut min_simple = usize::MAX;
    let mut min_complex = usize::MAX;
    for i in 0..counts.len() {
        let simple = cost_simple(&counts, i);
        if simple < min_simple {min_simple = simple;}

        let complex = cost_complex(&counts, i);
        if complex < min_complex {min_complex = complex;}
    }

    Ok((min_simple, min_complex))
}
